/*-------------------------------------------------------------------------+
| Copyright 2021 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.variability.service;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.service.IKernelIntrospectionSystemService;
import org.fortiss.tooling.kernel.service.base.EObjectAwareServiceBase;

/**
 * Implementation of the DependingElementProviderService which resolves semantic
 * and syntactic dependencies between model elements.
 * 
 * @author bayha
 */
public class DependingElementProviderService
		extends EObjectAwareServiceBase<IDependingElementProvider<? extends EObject>>
		implements IDependingElementProviderService {

	/** Singleton instance. */
	private static final DependingElementProviderService INSTANCE =
			new DependingElementProviderService();

	/** {@inheritDoc} */
	@Override
	public List<EObject> getDependingElements(EObject obj) {
		List<EObject> res = new ArrayList<EObject>();

		List<IDependingElementProvider<? extends EObject>> registeredHandlers =
				getRegisteredHandlers(obj.getClass());

		if(registeredHandlers != null) {
			for(IDependingElementProvider<?> p : registeredHandlers) {
				res.addAll(p.getDependingElements(obj));
			}
		}

		return res;
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		return "Depending Element Provider Service";
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionDescription() {
		return "Service providing elements which depend on a given element. " +
				"This service is used by the variability mechanisms in order to generate variants.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return "provider";
	}

	/** {@inheritDoc} */
	@Override
	public void startService() {
		IKernelIntrospectionSystemService.getInstance().registerService(this);
	}

	/**
	 * Retrieves the singleton instance.
	 * 
	 * @return Singleton instance.
	 */
	public static DependingElementProviderService getInstance() {
		return INSTANCE;
	}

	/** {@inheritDoc} */
	@Override
	public void registerDependingElementProvider(
			IDependingElementProvider<? extends EObject> provider, Class<? extends EObject> cls) {
		addHandler(cls, provider);
	}
}
